import React, { PropTypes } from 'react'
import { css } from 'glamor'
import theme from './theme'
import cx from 'classnames'
import {
  withSpacing,
  withClearfix,
  withBackground,
  withBackgroundColor,
  withTextAlign,
  withShadow,
  withFlex,
  withDisplay,
  withRounded,
  withPosition,
  withOverflow,
  withDimensions,
  withBorders,
  withTruncate,
  withTextStyles,
  withTransform
} from '../enhancers'

const n = 'ui-lib-reset'

// Add these styles globally until glamor properly supports &:matches

css.global(`article.${n}, aside.${n}, details.${n}, figcaption.${n}, figure.${n}, footer.${n}, header.${n}, hgroup.${n}, menu.${n}, nav.${n}, section.${n}`, {
  display: 'block'
})

css.global(`ol.${n}, ul.${n}`, {
  listStyle: 'none'
})

css.global(`bloquote.${n}, q.${n}`, {
  quotes: 'none'
})

css.global(`table.${n}`, {
  borderCollapse: 'collapse',
  borderSpacing: '0'
})

css.global(`input.${n}, textarea.${n}`, {
  border: 'none',
  outline: 'none'
})

css.global(`strong.${n}`, {
  fontWeight: '500'
})

css.global(`em.${n}`, {
  fontStyle: 'italic'
})

css.global(`*.${n}, *.${n}:before, *.${n}:after`, {
  boxSizing: 'border-box'
})

const cssBaseStyles = css({
  margin: '0',
  padding: '0',
  border: '0',
  fontSize: '100%',
  lineHeight: '1',
  verticalAlign: 'baseline',
  MozOsxFontSmoothing: 'grayscale',
  WebkitFontSmoothing: 'antialiased',
  fontSmoothing: 'antialiased',
  fontFamily: theme.fonts.sans,
  fontSize: theme.textStyles.body1.fontSize,
  fontWeight: theme.textStyles.body1.fontWeight,
  letterSpacing: theme.textStyles.body1.letterSpacing
})

const Box = ({
  is,
  className,
  theme,
  ...props
}) => React.createElement(is, { className: cx('ui-lib-reset', `${cssBaseStyles}`, className), ...props })

Box.propTypes = {
  is: PropTypes.oneOfType([
    PropTypes.string,
    PropTypes.element,
    PropTypes.func
  ])
}

Box.defaultProps = {
  is: 'div'
}

Box.displayName = 'Box'

// Decorate Box with higher order components
export default [
  withBackground,
  withBackgroundColor,
  withSpacing,
  withShadow,
  withClearfix,
  withTextAlign,
  withRounded,
  withTextStyles,
  withFlex,
  withDisplay,
  withPosition,
  withOverflow,
  withDimensions,
  withTransform,
  withBorders,
  withTruncate
].reduce((comp, hoc) => hoc(comp), Box)
